#!/usr/bin/env bash

[[ ${BURR_INPUT_LOADED:-} -eq 1 ]] && return

githubUser=martingoldie
branchContainer=corecommerce
defaultTracking=master
tracking=$defaultTracking
origin=fork

#cd


while getopts ":p:" opt; do
    case $opt in
        p)
            branchContainer=$OPTARG
        ;;

        \?)
            echo "Invalid option: -$OPTARG" >&2
            exit 1
        ;;

        :)
            echo "Option -$OPTARG requires an argument." >&2
            exit 1
        ;;
    esac
done

installHotfix(){
    installType=hotfix
    autoType=true
    tracking=master
    autoTracking=true

    installQuestions

    installCoreCommerce

    if [ -z "$installType" ]; then
        database=cc_local_default
    fi

    printf "$(tput setaf 3)You are now ready to install:\nInstall URL: http://localhost:8080/${branch}/install/index.php\nDatabase:$database\n$(tput sgr0)"
}

installFeature(){
    installType=feature
    autoType=true
    tracking=development
    autoTracking=true

    installQuestions

    installCoreCommerce

    if [ -z "$installType" ]; then
        database=cc_local_default
    fi

    printf "$(tput setaf 3)You are now ready to install:\nInstall URL: http://localhost:8080/${branch}/install/index.php\nDatabase:$database\n$(tput sgr0)"
}

installMaster(){
    installType=master
    autoType=true
    tracking=master
    autoTracking=true

    installCoreCommerce

    if [ -z "$installType" ]; then
        database=cc_local_default
    fi

    printf "$(tput setaf 3)You are now ready to install:\nInstall URL: http://localhost:8080/${branch}/install/index.php\nDatabase:$database\n$(tput sgr0)"

}

installDevelopment(){
    installType=development
    autoType=true
    tracking=development
    autoTracking=true

    installCoreCommerce
}

installAdvanced(){
    installQuestions
    installCoreCommerce
    addDatabase

    if [ -z "$installType" ]; then
        database=cc_local_default
    fi

    printf "$(tput setaf 3)You are now ready to install:\nInstall URL: http://localhost:8080/${branch}/install/index.php\nDatabase:$database\n$(tput sgr0)"
}

installCoreCommerce(){
    # $installType is directory name relative to calling script or relative to path indicated by coreinstall -p.
    # Use . for same directory
    # $issue is branch name: CC-[JIRA issue number]-description

    if [ -n "$issue" ]; then
        branch=${installType-default}\/${issue}
        branchDirectory=${installType-default}\/${issue}
    elif [ -z "$installType" ]; then
        branch=master
        branchDirectory=default
    else
        branch=${installType-master}
        branchDirectory=${branch}
    fi
    echo ${branchContainer} ${branch}
    pwd
    if [ ! -f Vagrantfile ] && [ ${branchContainer} == corecommerce ]; then
        echo -ne "$(tput setaf 2)\n***You aren't in the root of your Vagrant install.***$(tput sgr0)\n\nRemember you can override the install path with coreinstall -p /path/here/.\nAre you sure you want to continue?\n"
        while read -r -n 1 -s answer; do
            if [[ $answer = [Yy] ]]; then
                break
            elif [[ $answer = [Nn] ]]; then
                echo "Cancelling"
                exit 1
            fi
        done
    fi

    if [ -L $branchContainer/$branchDirectory/.git ] || [ -f $branchContainer/$branchDirectory/.git ] || [ -d $branchContainer/$branchDirectory/.git ]; then
        echo -ne "\n*** Git repo already exists at $branchContainer/$branchDirectory/ ***\n\nWould you like to overwrite it?\n[y]Yes\n[n]No(Cancel)\n:"
        while read -r -n 1 answer; do
            if [[ $answer = [Yy] ]]; then
                echo -ne "\n\n$(tput setaf 5)*** ARE YOU SURE? ***\n$(tput setaf 3)This will delete all files in $branchContainer/$branchDirectory/ and replace them with a new git clone.\n\n[y]Yes\n[n]No(Cancel)\n:$(tput sgr0)"
                read -r confirm
                case "$confirm" in
                    y|Y )
                        rm -rf ${branchContainer-FailIfEmpty}/${branchDirectory-FailIfEmpty}
                        mkdir ${branchContainer-FailIfEmpty}/${branchDirectory-FailIfEmpty}
                    ;;
                    [yY][Ee][Ss] )
                        rm -rf ${branchContainer-FailIfEmpty}/${branchDirectory-FailIfEmpty}
                        mkdir ${branchContainer-FailIfEmpty}/${branchDirectory-FailIfEmpty}
                    ;;
                    n|N )
                        echo "Cancelling"
                        exit 1
                    ;;
                    * )
                        echo -ne "Invalid Answer. Please use choose [y] or [n].\n[y]Yes\n[n]No(Cancel)\n:"
                        continue
                    ;;
                esac
                echo "Cloning from GitHub..."
                break
            elif [[ $answer = [Nn] ]]; then
                echo "Cancelling"
                exit 1
            else
                echo -ne "Invalid Answer. Please use choose [y] or [n].\n[y]Yes\n[n]No(Cancel)\n:"
            fi
        done
    fi

    if [ -f $branchContainer/$branchDirectory/.DS_Store ]; then
        rm -f $branchContainer/${branchDirectory}/.DS_Store
    fi

    if [ ! -d $branchContainer/${branchDirectory}/.git ] && [ -f $branchContainer/${branchDirectory}/.gitkeep ]; then
        rm -f $branchContainer/${branchDirectory}/.gitkeep
    fi

    if [ ! -d $branchContainer ]; then
        mkdir -p $branchContainer
    fi

    git clone git@github.com:${githubUser}/CoreCommerce.git ${branchContainer}/${branchDirectory}
    cd ${branchContainer}/${branchDirectory}
    git remote rename origin "$origin"
    git remote add upstream git@github.com:CoreCommerce/CoreCommerce.git
    git fetch upstream
    git show-ref --verify --quiet refs/heads/"$branch"
    success=$?
    if [[ $success -eq 0 ]]; then
        printf "$branch branch already exists.\nSwitching to $branch branch.\n"
        git checkout $branch
        git pull upstream $tracking
    else
        printf "Creating $branch branch."
        git checkout -b $branch upstream/$tracking
    fi

    # If you want to set it to ignore permissions for all repos globally
    # git config --global core.filemode false
    # Otherwise for only this repo use the below line instead.
    git config core.fileMode false
    git push "$origin" $branch

    if [ ! -d .git ]; then
        printf "$(tput setaf 9)\n*** Cloning failed. ***\n\n"
    else
        ssh -t $(vagrant ssh-config | awk '{print " -o "$1"="$2}') localhost "cd /var/www/html/"${branchDirectory}";composer.phar install --dev;"
    fi
    cd -
}

askSure() {
    echo -n "Are you sure (Y/N or Cancel)? "

    while read -r -n 1 -s answer; do
        if [[ $answer = [YyNn] ]]; then
            [[ $answer = [Yy] ]] && retval=0
            [[ $answer = [Nn] ]] && retval=1
            break
        elif [[ $answer = [cC] ]]; then
            echo "Cancelling"
            exit 1
        fi
    done

    echo ''

    return $retval
}

installQuestions(){

    declare -a arg
    arg=(installType issue tracking);

    for argv in ${arg[@]}; do
        case $argv in
            installType)
                if [ -z "$installType" ] || [ "$autoType" != true ]; then
                    echo -ne "$(tput setaf 6)\nPLEASE ENTER THE INSTALL TYPE:$(tput setaf 7)\n\n EG. hotfix\n\n For consistency please use either hotfix or feature and pay attention to lowercase/uppercase. \n\n$(tput setaf 3)ISSUE TYPE:$(tput sgr0) "
                    read installType
                    if [ "$installType" == "hotfix" ]; then
                        tracking=master
                    fi
                    if [ "$installType" == "feature" ]; then
                        tracking=development
                    fi
                    if [ "$installType" == "master" ]; then
                        tracking=master
                    fi
                    if [ "$installType" == "development" ]; then
                        tracking=development
                    fi
                fi
            ;;

            issue)
                echo -ne "$(tput setaf 6)\nPLEASE ENTER THE JIRA ISSUE:$(tput setaf 7)\n\n EG. CC-1111\n\n For consistency please use same format as in JIRA and pay attention to lowercase/uppercase. [PROJECT_INITIALS]-[NUMBER] \n\n$(tput setaf 3)JIRA ISSUE:$(tput sgr0) "
                read issue
            ;;

            tracking)
                if [ -z "$tracking" ] || [ "$autoTracking" != true ]; then
                    echo -ne "$(tput setaf 6)\nPLEASE ENTER THE TRACKING BRANCH:$(tput setaf 7)\n\n EG. master\n\n Tracking branch is automatically set to track from upstream repo so only specify the branch. \n\n$(tput setaf 3)TRACKING BRANCH:$(tput sgr0) "
                    read tracking
                fi
            ;;
        esac
    done
    echo -ne "\nJust to confirm:\n"
    echo -ne "Install type: $installType\n"
    echo -ne "Issue: $issue\n"
    echo -ne "Tracking: upstream/$tracking\n"

    until askSure; do
        for argv in ${arg[@]}; do
            case $argv in

                installType)
                    if [ -z "$installType" ] || [ "$autoType" != true ]; then
                        echo -ne "$(tput setaf 6)\nPLEASE ENTER THE INSTALL TYPE:$(tput setaf 7)\n\n EG. hotfix\n\n For consistency please use either hotfix or feature and pay attention to lowercase/uppercase. \n\n$(tput setaf 3)ISSUE TYPE:$(tput sgr0) "
                        read installType
                        if [ "$installType" == "hotfix" ]; then
                            tracking=master
                        fi
                        if [ "$installType" == "feature" ]; then
                            tracking=development
                        fi
                        if [ "$installType" == "master" ]; then
                            tracking=master
                        fi
                        if [ "$installType" == "development" ]; then
                            tracking=development
                        fi
                    fi
                ;;

                issue)
                    echo -ne "$(tput setaf 6)\nPLEASE ENTER THE JIRA ISSUE:$(tput setaf 7)\n\n EG. CC-1111\n\n For consistency please use same format as in JIRA and pay attention to lowercase/uppercase. [PROJECT_INITIALS]-[NUMBER] \n\n$(tput setaf 3)JIRA ISSUE:$(tput sgr0) "
                    read issue
                ;;

                tracking)
                    if [ -z "$tracking" ] || [ "$autoTracking" != true ]; then
                        echo -ne "$(tput setaf 6)\nPLEASE ENTER THE TRACKING BRANCH:$(tput setaf 7)\n\n EG. master\n\n Tracking branch is automatically set to track from upstream repo so only specify the branch. \n\n$(tput setaf 3)TRACKING BRANCH:$(tput sgr0) "
                        read tracking
                    fi
                ;;
            esac
        done
        echo -ne "\nJust to confirm:\n"
        echo -ne "Install type: $installType\n"
        echo -ne "Issue: $issue\n"
        echo -ne "Tracking: upstream/$tracking\n"

    done

    branch=$installType\/$issue
}

mysqlQuestions(){
    declare -a arg
    arg=(dbName);

    for argv in ${arg[@]}; do
        case $argv in

            dbName)
                # if [ "dbName" = null ]; then
                echo -ne "$(tput setaf 6)\nPLEASE ENTER DATABASE NAME:$(tput setaf 7)\n\n EG. cc_local_1111. \n\n$(tput setaf 3)DATABASE NAME:$(tput sgr0) "
                read dbName
                #fi
            ;;
        esac
    done

    echo -ne "\nJust to confirm:\n"
    echo -ne "Database name: $dbName\n"

    until askSure; do
        for argv in ${arg[@]}; do
            case $argv in
                dbName)
                    echo -ne "$(tput setaf 6)\nPLEASE ENTER DATABASE NAME:$(tput setaf 7)\n\n EG. cc_local_1111. \n\n$(tput setaf 3)DATABASE NAME:$(tput sgr0) "
                    read dbName
                ;;
            esac
        done
        echo -ne "\nJust to confirm:\n"
        echo -ne "Database name: $dbName\n"
    done
    dbName=$dbName
}

addDatabase(){
    # @todo we should check if there is already a database with same name and ask to cancel or overwrite
    # @todo we should check to make sure there are no errors and the database has been created
    mysqlQuestions
    database=${dbName-blank}

    ssh -t $(vagrant ssh-config | awk '{print " -o "$1"="$2}') localhost "mysql -uroot -proot -e 'create database "$database";'"

    echo "Database $database created"
}

dropDatabase(){
    # @todo we should check to make sure there are no errors and the database has been dropped
    mysqlQuestions
    database=${dbName-blank}


                echo -ne "\n\n$(tput setaf 5)*** ARE YOU SURE? ***\n$(tput setaf 3)This will DELETE the database "$database"\n\n[y]Yes\n[n]No(Cancel)\n:$(tput sgr0)"
                read -r confirm
                case "$confirm" in
                    y|Y )
                            ssh -t $(vagrant ssh-config | awk '{print " -o "$1"="$2}') localhost "mysql -uroot -proot -e 'drop database "$database";'"
                    ;;
                    [yY][Ee][Ss] )
                            ssh -t $(vagrant ssh-config | awk '{print " -o "$1"="$2}') localhost "mysql -uroot -proot -e 'drop database "$database";'"
                    ;;
                    n|N )
                        echo "Cancelling"
                        exit 1
                    ;;
                    * )
                        echo -ne "Invalid Answer. Please use choose [y] or [n].\n[y]Yes\n[n]No(Cancel)\n:"
                        continue
                    ;;
                esac


    echo "Database $database dropped(deleted)"
}

showDatabases(){
    ssh -t $(vagrant ssh-config | awk '{print " -o "$1"="$2}') localhost "mysql -uroot -proot -e 'show databases;'"
}

comingSoon(){
    echo -e "Coming Soon..." && sleep 2
}

BURR_INPUT_LOADED=1
